
#ifndef GEPH_H
#define GEPH_H 
 
/* ----------------------------------------------------------------------
  (c) 2011 Geodetic Observatory Pecny, http://www.pecny.cz (gnss@pecny.cz)
      Research Institute of Geodesy, Topography and Cartography
      Ondrejov 244, 251 65, Czech Republic

  Purpose: implements ephemerides and navigation classes
  Version: $ Rev: $

  2011-01-10 /JD: created

  TODO: overwritting mode
        return first/last
        limit for number of data inclusions per satellites
-*/

#include "gdata/gdata.h"
#include "gutils/gsys.h"
#include "gutils/gtime.h"

using namespace std;
    
namespace gnut {

class t_geph : public t_gdata {

 public:
  t_geph();
  virtual ~t_geph();
  
  // pointers to support NULL if not requested!
  virtual int pos( const t_gtime& t, double  xyz[3], double var[3] = NULL, double vel[3] = NULL, bool chk_health = true ){ return -1; }  // [m]
  virtual int clk( const t_gtime& t, double*    clk, double*   var = NULL, double*  dclk = NULL, bool chk_health = true ){ return -1; }  // [s]
  virtual bool healthy() const { return true; }
  virtual int chk()      const { return -1; }
  virtual string str()   const { return ""; }
  virtual int print()    const { cout << str(); return 0; }

  void clear();
  bool valid();

  GSYS    gsys() const;                                          // GNSS system
  string  gsat() const;                                          // satellite number

  // POZDEJI JEN GSAT a vse pres MUTEX !!!
  string  sat()      const{ return _sat; }                       // satellite number  
  double  interval() const{ return _interval; }                  // get validity interval
  t_gtime epoch()    const{ return _epoch; }                     // reference epoch
  t_gtime begin()    const{ return _epoch - _interval/2; }       // beg of validity
  t_gtime end()      const{ return _epoch + _interval/2; }       // end of validity
   
 protected:
  virtual void _clear();
  virtual bool _valid() const;

  string            _sat;         // satellite number
  t_gtime           _epoch;       // reference epoch
  double            _interval;    // validity interval
   
 private:

};

} // namespace

#endif
