
/* ----------------------------------------------------------------------
 * G-Nut - GNSS software development library
 * 
  (c) 2011 Geodetic Observatory Pecny, http://www.pecny.cz (gnss@pecny.cz)
      Research Institute of Geodesy, Topography and Cartography
      Ondrejov 244, 251 65, Czech Republic

  This file is part of the G-Nut C++ library.
 
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License as
  published by the Free Software Foundation; either version 3 of
  the License, or (at your option) any later version.
 
  This library is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
  General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, see <http://www.gnu.org/licenses>.

-*/

#include <stdio.h>
#include <math.h>

#include "gdata/gobj.h"
#include "gutils/gtypeconv.h"

using namespace std;

namespace gnut {

// constructor
// ----------
t_gobj::t_gobj()
 :  _id(""),
    _name("")
{
  gtrace("t_gobj::constructor");
  id_type(OBJ);
// NOT NECESSARY 
//  t_gtriple eccnull(0.0, 0.0, 0.0);
//  _mapecc[t_gtime::first_time()] = eccnull;
  
   
}

/*
// copy constructor
// ----------
t_gobj::t_gobj(const t_gobj& obj)
 :  _id(obj.id()),
    _name(obj.name())
{
   gtrace("t_gobj::constructor");
  _type = obj.id_type();

  vector<t_gtime> vTIM = obj.ecc_id();
  vector<t_gtime>::iterator itTIM = vTIM.begin();
  while( itTIM != vTIM.end() ){
    this->ecc( *itTIM, obj.ecc( *itTIM ));
    itTIM++;
  }

   vTIM = obj.pcv_id();
  itTIM = vTIM.begin();
  while( itTIM != vTIM.end() ){
    this->pcv( *itTIM, obj.pcv( *itTIM ));
    itTIM++;
  }

   vTIM = obj.ant_id();
  itTIM = vTIM.begin();
  while( itTIM != vTIM.end() ){
    this->ant( *itTIM, obj.ant( *itTIM ));
    itTIM++;
  }
}
*/

// destructor
// ----------
t_gobj::~t_gobj()
{
  gtrace("t_gobj::destructor");
   
#ifdef BMUTEX
  boost::mutex::scoped_lock lock(_mutex);
#endif
  _gmutex.lock();

//_mapantpcv.clear();
  _mappcv.clear();
  _mapeccxyz.clear();
  _mapeccneu.clear();   
  _mapant.clear();
  _mapcrd.clear();   
//_params.clear();
  _gmutex.unlock();
}


// set id
// ----------
void t_gobj::id( string str )
{
   gtrace("t_gobj::it(str)");   
   
#ifdef BMUTEX
  boost::mutex::scoped_lock lock(_mutex);
#endif
  _gmutex.lock();
  _id = str;
  _gmutex.unlock();
}


// get name
// ----------
string t_gobj::id() const
{
   gtrace("t_gobj::id");   
   
#ifdef BMUTEX
  boost::mutex::scoped_lock lock(_mutex);
#endif
  _gmutex.lock();
  string tmp = _id;
  _gmutex.unlock(); return tmp;
}


// set name
// ----------
void t_gobj::name( string str )
{
   gtrace("t_gobj::name(str)");   
   
#ifdef BMUTEX
  boost::mutex::scoped_lock lock(_mutex);
#endif
 _gmutex.lock();
 _name = str;
 _gmutex.unlock();
}


// get name
// ----------
string t_gobj::name() const
{
   gtrace("t_gobj::name");   
   
#ifdef BMUTEX
  boost::mutex::scoped_lock lock(_mutex);
#endif
  _gmutex.lock();
  string tmp = _name;
  _gmutex.unlock(); return tmp;
}


// set description
// ----------
void t_gobj::desc( string str )
{
   gtrace("t_gobj::desc(str)");   
   
#ifdef BMUTEX
  boost::mutex::scoped_lock lock(_mutex);
#endif
  _gmutex.lock();
  _desc = str;
  _gmutex.unlock();  
}


// get description
// ----------
string t_gobj::desc() const
{
   gtrace("t_gobj::desc");   
   
#ifdef BMUTEX
  boost::mutex::scoped_lock lock(_mutex);
#endif
  _gmutex.lock();
  string tmp = _desc;
  _gmutex.unlock(); return tmp;
}


// set ecc offsets w.r.t. center of mass/reference point
// ----------
void t_gobj::eccxyz(const t_gtriple& ecc, const t_gtime& beg, const t_gtime& end )
{
   gtrace("t_gobj::eccxyz(...)");   
   
#ifdef BMUTEX
  boost::mutex::scoped_lock lock(_mutex);
#endif
  _gmutex.lock();

  t_gtriple zero(0.0, 0.0, 0.0);
  t_mapecc_xyz::iterator it =  _mapeccxyz.find(beg);

  if( !(beg < end) ){
     string lg = "Warning: " + _id + " not valid end time (end<beg) for eccentricity xyz";
    if( _log ) _log->comment(1,"gobj", lg);
    else               cerr << "gobj: " << lg << endl;
    _gmutex.unlock(); return;
  }
   
  // begin record
  if( it == _mapeccxyz.end() ){     // not exists
      _mapeccxyz[beg] = ecc;
     
  }else{                         // record exists
    if( it->first == t_gtime::last_time() ||
        it->second == zero ){

       _mapeccxyz[beg] = ecc;
    }else{
       string lg = "Warning: " + _id + " valid object record cannot be changed for eccentricity xyz";
       if( _log ) _log->comment(1,"gobj", lg);
       else               cerr << "gobj: " << lg << endl;
       _gmutex.unlock(); return;
    }
  }

  // control end of record (with new beg search)
  it = _mapeccxyz.find(beg); it++;
   
  // beg was last in map (add final empty record)
  if( it == _mapeccxyz.end() ){
    _mapeccxyz[end] = zero;

  }else{                                            // process end according to next value
    if( end < it->first ){                          // only if end is smaller then existing
      if( fabs(it->first - end) > 3600 ){           // significantly smaller!
        if( it->second == zero ) _mapeccxyz.erase(it); // remove obsolete empty record
        _mapeccxyz[end] = zero;
      
      }else{                                        // too close to next record
	string lg = "Warning: " + _id + " 'obj' end tied to the existing value " + end.str("%Y-%m-%d %H:%M:%S -> ") + it->first.str("%Y-%m-%d %H:%M:%S");
        if( _log ) _log->comment(1,"gobj",lg);
        else               cerr << "gobj: " << lg << endl;
      }
    }
  }

  // remove duplicated empty records
  t_mapecc_xyz::iterator itNEW = _mapeccxyz.begin();
  t_mapecc_xyz::iterator itOLD = itNEW;
  while( itOLD != _mapeccxyz.end() ){    
    if( ++itNEW != _mapeccxyz.end() ){
      if( ( itNEW->second == zero && itOLD->second == zero ) ||
          ( itNEW->first == t_gtime::last_time() ) )
      {
        _mapeccxyz.erase( itNEW++ );
      }
    }
    itOLD = itNEW;
  }
  _gmutex.unlock(); return;
}


// get ecc offsets (>=t) w.r.t. center of mass/reference point
// ----------
t_gtriple t_gobj::eccxyz(const t_gtime& t) const
{
   gtrace("t_gobj::eccyz");   
   
#ifdef BMUTEX
  boost::mutex::scoped_lock lock(_mutex);
#endif
  _gmutex.lock();

  t_gtriple ecc(0.0, 0.0, 0.0);
  t_mapecc_xyz::const_iterator it = _mapeccxyz.upper_bound(t);
  if( it == _mapeccxyz.begin() ){ _gmutex.unlock(); return ecc; }   // not found
  it--;
  t_gtriple tmp = it->second;
  _gmutex.unlock(); return tmp;
}

// set ecc offsets w.r.t. center of mass/reference point
// ----------
void t_gobj::eccneu(const t_gtriple& ecc, const t_gtime& beg, const t_gtime& end )
{
   gtrace("t_gobj::eccneu(...)");   
   
#ifdef BMUTEX
  boost::mutex::scoped_lock lock(_mutex);
#endif
  _gmutex.lock();

  t_gtriple zero(0.0, 0.0, 0.0);
  t_mapecc_neu::iterator it =  _mapeccneu.find(beg);

  if( !(beg < end) ){
    string lg = "Warning: " + _id + " not valid end time (end<beg) for eccentricity neu";
    if( _log ) _log->comment(1,"gobj", lg );
    else               cerr << "gobj: " << lg << endl;
    _gmutex.unlock(); return;
  }
   
  // begin record
  if( it == _mapeccneu.end() ){     // not exists
      _mapeccneu[beg] = ecc;
     
  }else{                         // record exists
    if( it->first == t_gtime::last_time() ||
        it->second == zero ){

       _mapeccneu[beg] = ecc;
    }else{
       string lg = "Warning: " + _id + " valid object record cannot be changed for eccentricity xyz";
       if( _log ) _log->comment(1,"gobj", lg );
       else               cerr << "gobj: " << lg << endl;
       _gmutex.unlock(); return;
    }
  }

  // control end of record (with new beg search)
  it = _mapeccneu.find(beg); it++;
   
  // beg was last in map (add final empty record)
  if( it == _mapeccneu.end() ){
    _mapeccneu[end] = zero;

  }else{                                            // process end according to next value
    if( end < it->first ){                          // only if end is smaller then existing
      if( fabs(it->first - end) > 3600 ){           // significantly smaller!
        if( it->second == zero ) _mapeccneu.erase(it); // remove obsolete empty record
        _mapeccneu[end] = zero;
      
      }else{                                        // too close to next record
	string lg = "Warning: " + _id + " 'obj' end tied to the existing value " + end.str("%Y-%m-%d %H:%M:%S -> ") + it->first.str("%Y-%m-%d %H:%M:%S");
        if( _log ) _log->comment(1,"gobj",lg );
        else               cerr << "gobj: " << lg << endl;
      }
    }
  }

  // remove duplicated empty records
  t_mapecc_neu::iterator itNEW = _mapeccneu.begin();
  t_mapecc_neu::iterator itOLD = itNEW;
  while( itOLD != _mapeccneu.end() ){    
    if( ++itNEW != _mapeccneu.end() ){
      if( ( itNEW->second == zero && itOLD->second == zero ) ||
          ( itNEW->first == t_gtime::last_time() ) )
      {
        _mapeccneu.erase( itNEW++ );
      }
    }
    itOLD = itNEW;
  }
  _gmutex.unlock(); return;
}


// get ecc offsets (>=t) w.r.t. center of mass/reference point
// ----------
t_gtriple t_gobj::eccneu(const t_gtime& t) const
{
   gtrace("t_gobj::eccneu");   
   
#ifdef BMUTEX
  boost::mutex::scoped_lock lock(_mutex);
#endif
  _gmutex.lock();

  t_gtriple ecc(0.0, 0.0, 0.0);
  t_mapecc_neu::const_iterator it = _mapeccneu.upper_bound(t);
  if( it == _mapeccneu.begin() ){ _gmutex.unlock(); return ecc; }  // not found
  it--;
  t_gtriple tmp = it->second;
  _gmutex.unlock(); return tmp;
}

// set object position
// ----------
void t_gobj::crd(const t_gtriple& crd, const t_gtime& beg, const t_gtime& end )
{
  gtrace("t_gobj::crd(t_gtriple, epo)");
   
#ifdef BMUTEX
  boost::mutex::scoped_lock lock(_mutex);
#endif
  _gmutex.lock();

  t_gtriple zero(0.0, 0.0, 0.0);
  t_mapcrd::iterator it =  _mapcrd.find(beg);

  if( !(beg < end) ){
    string lg = "Warning: " + _id + " not valid end time (end<beg) for coordinates";
    if( _log ) _log->comment(1,"gobj",lg );
    else               cerr << "gobj: " << lg << endl;
    _gmutex.unlock(); return;
  }

  // begin record
  if( it == _mapcrd.end() ){     // not exists
      _mapcrd[beg] = crd;
  }else{                         // record exists
    if( it->first == t_gtime::last_time() ||
        it->second == zero ){
       _mapcrd[beg] = crd;
    }else{
       string lg = "Warning: " + _id + " valid object record cannot be changed (coordinates)";
       if( _log ) _log->comment(1,"gobj",lg );
       else               cerr << "gobj: " << lg << endl;
       _gmutex.unlock(); return;
    }
  }
  // control end of record (with new beg search)
  it = _mapcrd.find(beg); it++;
   
  // beg was last in map (add final empty record)
  if( it == _mapcrd.end() ){
    _mapcrd[end] = zero;

  }else{                                            // process end according to next value
    if( end < it->first ){                          // only if end is smaller then existing
      if( fabs(it->first - end) > 3600 ){           // significantly smaller!
        if( it->second == zero ) _mapcrd.erase(it); // remove obsolete empty record
        _mapcrd[end] = zero;
      
      }else{                                        // too close to next record
	string lg = "Warning: " + _id + " 'obj' end tied to the existing value " + end.str("%Y-%m-%d %H:%M:%S -> ") + it->first.str("%Y-%m-%d %H:%M:%S");
        if( _log ) _log->comment(1,"gobj",lg );
        else               cerr << "gobj: " << lg << endl;
      }
    }
  }

  // remove duplicated empty records
  t_mapcrd::iterator itNEW = _mapcrd.begin();
  t_mapcrd::iterator itOLD = itNEW;
  while( itOLD != _mapcrd.end() ){    
    if( ++itNEW != _mapcrd.end() ){
      if( ( itNEW->second == zero && itOLD->second == zero ) ||
          ( itNEW->first == t_gtime::last_time() ) )
      {
        _mapcrd.erase( itNEW++ );
      }
    }
    itOLD = itNEW;
  }

  _gmutex.unlock(); return;
}

// get object position
// ----------
t_gtriple t_gobj::crd(const t_gtime& t) const
{
   gtrace("t_gobj::crd");
   
#ifdef BMUTEX
  boost::mutex::scoped_lock lock(_mutex);
#endif
  _gmutex.lock();

  t_gtriple crd(0.0, 0.0, 0.0);
  t_mapcrd::const_iterator it = _mapcrd.upper_bound(t);
  if( it == _mapcrd.begin() ){ _gmutex.unlock(); return crd; }   // not found
  it--;
  t_gtriple tmp = it->second;
  _gmutex.unlock(); return tmp;
}

// set pcv element
// ----------
void t_gobj::pcv(shared_ptr<t_gpcv> pcv, const t_gtime& beg, const t_gtime& end)
{
   gtrace("t_gobj::pcv(...)");   
   
#ifdef BMUTEX
  boost::mutex::scoped_lock lock(_mutex);
#endif
  _gmutex.lock();

  this->_pcv(pcv, beg, end);
  _gmutex.unlock();
}


// set pcv element (protected)
// ----------
void t_gobj::_pcv(shared_ptr<t_gpcv> pcv, const t_gtime& beg, const t_gtime& end)
{
   gtrace("t_gobj::_pcv");   
   
  // ------------------------------------------------------------------
  // ------------------------------------------------------------------
  // _mappcv[beg] = pcv;  // _mappcv[t] = pcv->pointer(); !!!! ???????? 
  // ------------------------------------------------------------------
  // ------------------------------------------------------------------
  
  /* t_mapantpcv::iterator itant = _mapantpcv.find(ant); */
  /* if (itant == _mapantpcv.end()){ */
  /*    _mapantpcv[ant][beg] = pcv; */
  /* }else{ */
  /*   t_mappcv mappcv = itant->second; */
     
    t_mappcv::iterator it = _mappcv.find(beg);

    if( !(beg < end) ){
      string lg = "Warning: " + _id + " not valid end time (end<beg) for PCV";
      if( _log ) _log->comment(1,"gobj",lg );
      else               cerr << "gobj: " << lg << endl;
      return;
    }
   
    // begin record
    if( it == _mappcv.end() ){     // not exists
      _mappcv[beg] = pcv;        // ?? POINTER ??
    
    // record exists
    }else if( it->first == t_gtime::last_time() || it->second ){
      _mappcv[beg] = pcv;        // ?? POINTER ??

    }else{
      string lg = "Warning: " + _id + " valid object record cannot be changed (PCV)";
      if( _log ) _log->comment(1,"gobj",lg );
      else               cerr << "gobj: " << lg << endl;
      return;
    }
     
    // control end of record (with new beg search)
    it = _mappcv.find(beg); it++;
     
    // beg was last in map (add final empty record)
    if( it == _mappcv.end() ){
      _mappcv[end] = _pcvnull;
  
    }else{                                            // process end according to next value
      if( end < it->first ){                          // only if end is smaller then existing
        if( fabs(it->first - end) > 3600 ){           // significantly smaller!
          if( it->second == 0 ) _mappcv.erase(it); // remove obsolete empty record
          _mappcv[end] = _pcvnull;
        
        }else{                                        // too close to next record
   	  string lg = "Warning: " + _id + " 'obj' end tied to the existing value " + end.str("%Y-%m-%d %H:%M:%S -> ") + it->first.str("%Y-%m-%d %H:%M:%S");
          if( _log ) _log->comment(1,"gobj",lg );
	  else               cerr << "gobj: " << lg << endl;
        }
      }
    }
  
    // remove duplicated empty records
    t_mappcv::iterator itNEW = _mappcv.begin();
    t_mappcv::iterator itOLD = itNEW;
    while( itOLD != _mappcv.end() ){    
      if( ++itNEW != _mappcv.end() ){
        if( ( itNEW->second && itOLD->second ) ||
            ( itNEW->first == t_gtime::last_time() ) )
        {
          _mappcv.erase( itNEW++ );
        }
      }
      itOLD = itNEW;
    }     
//  }      
}


// get pcv element (>=t)
// ----------
shared_ptr<t_gpcv> t_gobj::pcv(const t_gtime& t) const
{  
   gtrace("t_gobj::pcv(t)");   
   
#ifdef BMUTEX
  boost::mutex::scoped_lock lock(_mutex);
#endif
  _gmutex.lock();
  shared_ptr<t_gpcv> tmp = _pcv(t);
  _gmutex.unlock(); return tmp;
}

// get pcv element (>=t) (protected)
// ----------
shared_ptr<t_gpcv> t_gobj::_pcv(const t_gtime& t) const
{  
   gtrace("t_gobj::_pcv(t)");   
   
  shared_ptr<t_gpcv> pcv;
   
  string ant;   
  t_mapant::const_iterator it = _mapant.upper_bound(t);
  if( it == _mapant.begin() ) {
     ant = "";  // antenna not found

     return _pcvnull;
  }else{
    ant = (--it)->second;     
  }

  t_mappcv::const_iterator it2 = _mappcv.upper_bound(t);
  if( it2 == _mappcv.begin() ) {

     return _pcvnull;
  }else{	
    pcv = (--it2)->second;
  }

  if(pcv->pcvkey().compare(ant) != 0){
#ifdef DEBUG     
     cout << "gobj RESET: ant [" << ant << "] [" << pcv->pcvkey() << " "  << t.str_ymdhms(" epo:") << "]\n";
#endif     
     return _pcvnull;
  }
   
  return pcv;
}


// set antenna name
// ----------
void t_gobj::ant(string ant, const t_gtime& beg, const t_gtime& end)
{
   gtrace("t_gobj::and(...)");   
   
#ifdef BMUTEX
  boost::mutex::scoped_lock lock(_mutex);
#endif
  _gmutex.lock();

  t_mapant::iterator it =  _mapant.find(beg);

  if( !(beg < end) ){
    string lg = "Warning: " + _id + " not valid end time (end<beg) for antenna";
    if( _log ) _log->comment(1,"gobj",lg );
    else               cerr << "gobj: " << lg << endl;
    _gmutex.unlock(); return;
  }

  // begin record
  if( it == _mapant.end() ){     // not exists
      _mapant[beg] = ant;
     
  }else{                         // record exists
    if( it->first == t_gtime::last_time() ||
        it->second.empty() ){

       _mapant[beg] = ant;
    }else{
       string lg = "Warning: " + _id + " valid object record cannot be changed (antenna)";
       if( _log ) _log->comment(1,"gobj",lg );
//       else               cerr << "gobj: " << lg << endl;
       _gmutex.unlock(); return;
    }
  }

  // control end of record (with new beg search)
  it = _mapant.find(beg); it++;
   
  // beg was last in map (add final empty record)
  if( it == _mapant.end() ){
    _mapant[end] = "";

  }else{                                            // process end according to next value
    if( end < it->first ){                          // only if end is smaller then existing
      if( fabs(it->first - end) > 3600 ){           // significantly smaller!
        if( it->second.empty() ) _mapant.erase(it); // remove obsolete empty record
        _mapant[end] = "";
      
      }else{                                        // too close to next record
 	string lg = "Warning: " + _id + " 'obj' end tied to the existing value " + end.str("%Y-%m-%d %H:%M:%S -> ") + it->first.str("%Y-%m-%d %H:%M:%S");
        if( _log ) _log->comment(1,"gobj",lg );
        else               cerr << "gobj: " << lg << endl;
      }
    }
  }

  // remove duplicated empty records
  t_mapant::iterator itNEW = _mapant.begin();
  t_mapant::iterator itOLD = itNEW;
  while( itOLD != _mapant.end() ){    
    if( ++itNEW != _mapant.end() ){
      if( ( itNEW->second.empty() && itOLD->second.empty() ) ||
          ( itNEW->first == t_gtime::last_time() ) )
      {
        _mapant.erase( itNEW++ );
      }
    }
    itOLD = itNEW;
  }

  _gmutex.unlock(); return;
}


// get antenna name (>=t)
// ----------
string t_gobj::ant(const t_gtime& t) const
{  
   gtrace("t_gobj::ant()");   
   
#ifdef BMUTEX
  boost::mutex::scoped_lock lock(_mutex);
#endif
  _gmutex.lock();
  string tmp = this->_ant(t);
  _gmutex.unlock(); return tmp;
}

// get antenna name (>=t) (protected)
// ----------
string t_gobj::_ant(const t_gtime& t) const
{  
   gtrace("t_gobj::_ant(t)");   
   
  t_mapant::const_iterator it = _mapant.upper_bound(t);
  if( it == _mapant.begin() ) return "";  // not found
  it--;
  return it->second;
}

     
// get time tags
// ----------
/*vector<t_gtime> t_gobj::ecc_id() const
{
  vector<t_gtime> tmp;
  t_mapecc_xyz::const_iterator itMAP = _mapeccxyz.begin();
  while( itMAP != _mapecc.end() ){
    tmp.push_back( itMAP->first );
    itMAP++;
  }
  return tmp;
}
*/
     
// get time tags
// ----------
vector<t_gtime> t_gobj::pcv_id() const
{      
   gtrace("t_gobj::pcv_id");   
   
#ifdef BMUTEX
  boost::mutex::scoped_lock lock(_mutex);
#endif
  _gmutex.lock();

  vector<t_gtime> tmp;
//  t_mapantpcv::const_iterator itant = _mapantpcv.find(ant);
   
//  if (itant == _mapantpcv.end()){
//     return tmp;
//  }else{
  t_mappcv::const_iterator itMAP = _mappcv.begin();
  while( itMAP != _mappcv.end() ){
    tmp.push_back( itMAP->first );
    itMAP++;
  }
  _gmutex.unlock(); return tmp;     
//  }
   
}

     
// get time tags
// ----------
vector<t_gtime> t_gobj::ant_id() const
{
   gtrace("t_gobj::ant_id");   
   
#ifdef BMUTEX
  boost::mutex::scoped_lock lock(_mutex);
#endif
  _gmutex.lock();
  vector<t_gtime> tmp = this->_ant_id();
  _gmutex.unlock(); return tmp;   
}

// get time tags (protected)
// ----------
vector<t_gtime> t_gobj::_ant_id() const
{   
   gtrace("t_gobj::_ant_id");   
   
  vector<t_gtime> tmp;
  t_mapant::const_iterator itMAP = _mapant.begin();
  while( itMAP != _mapant.end() ){
    tmp.push_back( itMAP->first );
    itMAP++;
  }
  return tmp;
}

// set mappcv for all t_gobj
// -----------------------
void t_gobj::sync_pcv(t_gallpcv* pcvs)
{
   gtrace("t_gobj::sync_pcv");   
   
  if( pcvs == 0 ) return;

#ifdef BMUTEX
  boost::mutex::scoped_lock lock(_mutex);
#endif
  _gmutex.lock();

  vector<t_gtime> vant = this->_ant_id();

  if( vant.size() == 0 ){  // ant in obj is not set
    if( _log && _log->verb() >= 2 ) _log->comment(2,"gobj","Warning: " + _id + " ant name not set: " );
    else                                    cerr << "gobj - Warning: " + _id + " ant name not set:" << endl;

  }else{
     for( vector<t_gtime>::iterator it = vant.begin(); it != vant.end(); it++ ){

       // set pcv for all antennas
       string ant = this->_ant(*it);
       t_gtime epo = *it;
       shared_ptr<t_gpcv> gpcv = pcvs->gpcv(ant, "*", epo);
       if( gpcv != _pcvnull ) this->_pcv(gpcv, epo);

#ifdef DEBUG
       cout << " gobj.cpp - sync for ant " << ant << "  pcv found [" << gpcv << "]" << endl;
#endif
     }
  }   
  _gmutex.unlock(); return;
}


// check consistency
// ----------
map<string, pair<string,string>> t_gobj::compare(shared_ptr<t_gobj> gobj, const t_gtime& tt)
{
  gtrace("t_gobj::compare");
   
// #ifdef BMUTEX
//  boost::mutex::scoped_lock lock(_mutex);
//#endif

//_gmutex.lock();

  map<string, pair<string,string>> m_res;
  if( gobj->id()       != this->id()       ){ m_res["ID"]          = make_pair(this->id(),      gobj->id());       }
  if( gobj->name()     != this->name()     ){ m_res["NAME"]        = make_pair(this->name(),    gobj->name());     }
//if( gobj->desc()     != this->desc()     ){ m_res["DESCRIPTION"] = make_pair(this->desc(),    gobj->desc());     }

  string old, alt;
  old = trim(      ant(tt));
  alt = trim(gobj->ant(tt));
  if( gobj->ant(tt)    != this->ant(tt)    ){ m_res["ANTENNA"]     = make_pair(old, alt); }
   
//if( gobj->crd(tt)    != this->crd(tt)    ){ m_res["CRD_XYZ"]     = make_pair(this->crd(tt),   gobj->crd(tt));    }
//if( gobj->eccxyz(tt) != this->eccxyz(tt) ){ m_res["ECC_XYZ"]     = make_pair(this->eccxyz(tt),gobj->eccxyz(tt)); }
//if( gobj->eccneu(tt) != this->eccneu(tt) ){ m_res["ECC_NEU"]     = make_pair(this->eccneu(tt),gobj->eccneu(tt)); }

//_gmutex.unlock(); 
  return m_res;
}


// operator for sorting
// ----------
bool t_gobj::operator<(const t_gobj& t) const 
{ 
 return _name < t.name();
}
   

// operator for sorting
// ----------
bool t_gobj::operator==(const t_gobj& t) const
{
 return  _name == t.name();
}
   
} // namespace
