
/* ----------------------------------------------------------------------
 * G-Nut - GNSS software development library
 * 
  (c) 2011 Geodetic Observatory Pecny, http://www.pecny.cz (gnss@pecny.cz)
      Research Institute of Geodesy, Topography and Cartography
      Ondrejov 244, 251 65, Czech Republic

  This file is part of the G-Nut C++ library.
 
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License as
  published by the Free Software Foundation; either version 3 of
  the License, or (at your option) any later version.
 
  This library is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
  General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, see <http://www.gnu.org/licenses>.

-*/

#include <cstring>

#include "gio/gfile.h"

using namespace std;

namespace gnut {

// constructor
// ----------
t_gfile::t_gfile()
  : t_gio(),
    _file("")
{
  _size = FILEBUF_SIZE;
}


// destructor
// ----------
t_gfile::~t_gfile()
{}


// set path
// ----------
int t_gfile::path( string str ){

  if( str == "" ) return -1;

  size_t idx1 = 0, idx2 = 0;
  string tmp = "file://";

  // check if file
  idx2 = str.find_first_of(tmp,0);
  if( idx2 == string::npos ){
    if( _log ) _log->comment(0, "warning: path does not contain file://dir/name  [check file://]");
    return -1; 
  }

  // file path
  idx1 = idx2+tmp.length();
  idx2 = str.length();
  if( idx2 != string::npos and idx2 > idx1  ){
      
    string name = str.substr(idx1,idx2-idx1);
    _file.mask(name);

    ostringstream ltmp;
    ltmp << " file: " << int(idx1) << ":" << int(idx2) << " = " << name << endl;
    if( _log ) _log->comment(3, ltmp.str());
    
  }else{
    if( _log ) _log->comment(0, "warning: path does not contain file://dir/name  [check file://]");
    return -1; 
  }

  return 1;
}


// get path
// ----------
string t_gfile::path(){

  return "file://" + _file.mask();
}


// init read function (read header)
// ----------
int t_gfile::init_read(){
  if( ! _coder ){ return 0; }

  char* loc_buff = new char[FILEHDR_SIZE];

#ifdef DEBUG
  cout << "HEADER START " << _file.mask() << "\n"; cout.flush();
#endif

  int nbytes = 0;
  vector<string>  errmsg;
//  while( ( nbytes = _gio_read( loc_buff, FILEHDR_SIZE ) ) >= 0 and _stop != 1 ){ // JD 2014-10-25
  while( ( nbytes = _gio_read( loc_buff, FILEHDR_SIZE ) ) > 0 and _stop != 1 ){
     
#ifdef DEBUG
      cout << "GIO_HEAD_READ:"; 
      for(int i = 0; i<nbytes ; i++){ cout << loc_buff[i]; } 
      cout << ":END\n";
#endif
     
    if( _coder->decode_head( loc_buff, nbytes, errmsg ) < 0 ) break;
  }

#ifdef DEBUG
  cout << "HEADER END " << _file.mask() << "\n";
#endif

  delete [] loc_buff;
  return 1; 
}


// init write function (write header)
// ----------
int t_gfile::init_write(){
  if( ! _coder ){ return 0; }
   
  char* loc_buff = new char[FILEHDR_SIZE];

#ifdef DEBUG
  cout << "HEADER START " << _file.mask() << endl; cout.flush();
#endif
  vector<string> errmsg;
   
  int nbytes = 0;
  do{
    if( ( nbytes = _coder->encode_head( loc_buff, FILEHDR_SIZE, errmsg ) ) < 0 ) break;
  }while( (nbytes > 0) && (_gio_write( loc_buff, nbytes ) > 0) && _stop != 1  );
   
#ifdef DEBUG
  cout << "HEADER END " << _file.mask() << endl; cout.flush();
#endif
   
  delete [] loc_buff;
      
  return 1; 
}


// read data
// ----------
int t_gfile::_gio_read( char* buff, int size )
{     
  if( _file.mask() == ""   ) return -1;
  int irc =  _file.read(buff,size);
  if( irc == 0 && _file.eof() ) return -1;
  return irc;

//  int nbytes = 0
//  nbytes = _file.read(buff,size);
//  if( _file.eof() || nbytes == -1 ){
//    if( nbytes == 0 ) nbytes = -1;
//    }
//  }
//  return nbytes;
}


// send data
// ----------
int t_gfile::_gio_write( const char* buff, int size )
{
  if( _file.mask() == ""   ) return 0;

//  int irc =
  _file.write( buff, size );

//  if( irc ) return 0;
  return size;
}


// common function for file close
// ----------
int t_gfile::_stop_common()
{ 
  return t_gio::_stop_common();
}

} // namespace
