
/* ----------------------------------------------------------------------
 * G-Nut - GNSS software development library
 * 
  (c) 2011 Geodetic Observatory Pecny, http://www.pecny.cz (gnss@pecny.cz)
      Research Institute of Geodesy, Topography and Cartography
      Ondrejov 244, 251 65, Czech Republic

  This file is part of the G-Nut C++ library.
 
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License as
  published by the Free Software Foundation; either version 3 of
  the License, or (at your option) any later version.
 
  This library is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
  General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, see <http://www.gnu.org/licenses>.

-*/

#include <stdlib.h>
#include <iostream>
#include <iomanip>

#include "gmodels/ginterp2d.h"
#include "../newmat/newmatio.h"

using namespace std;

namespace gnut {   

// constructor
// ----------
t_ginterp2d::t_ginterp2d(){}

// destructor
// ----------
t_ginterp2d::~t_ginterp2d(){}


// linear interpolation
int t_ginterp2d::linear(map<double, double>& data, double val, double& fval)
{
   double min = data.begin()->first;
   double max = data.rbegin()->first;

   if (val < min || val > max) {
      cerr << "Warning: extrapolation is not allowed! x<" << min << "," << max << ") for val=" << val << endl;
      return -1;
   }

   map<double, double>::iterator it2, it1;   
   it2 = data.lower_bound(val);   
   it1 = it2;
   it1--;
   
   Matrix x(2,2), y(2,1);
   x << it1->first << 1
     << it2->first << 1;
   y << it1->second 
     << it2->second;
   
   Matrix coef(2,1);
   coef = x.i()*y;
   
   fval = coef(1,1)*val + coef(2,1);

#ifdef DEBUG   
   cout << "t_ginterp2d::linear: " << x << " " << y << " " << "x = " << val << " fx = " << fval << endl;
#endif
   return 1;
}

} // namespace
